
type sv_t =
  | Pos of int *int
  | Int of int

module TML = Tml.Gen_TML(struct type t = sv_t end)

open TML
open Graphics

let h = 10         (* height of planche *)
let x0 = 60        (* lower left ...    *)
and y0 = 110       (* ... corner        *)
and ystep = 40    
and gen_itv = 40   (* ball generation interval *)
let radius=9       (* of ball *) 
let pts_radius = 5


let r2 = radius*2
let rr = radius + pts_radius

let yinit = y0+h*ystep
let xinit = x0+h*ystep

let nb_cnts = h+1
let nb_pts = h*(h+1)/2

let pts = Array.create nb_pts (0,0)

let make_pts () =
  let i = ref 0 in
  for y = 1 to h do
    let doit = ref true in
    for x = y to 2*h-y do
      if !doit then begin
	pts.(!i) <- (x0+x*ystep, y0+(y-1)*ystep);
	incr i
      end;
      doit := not !doit
    done;
  done

let draw_pts () =
  Array.iter (fun (x,y) -> fill_circle x y pts_radius) pts

let draw_ball x y = draw_circle x y radius

let draw_cnt i v = moveto (i*ystep*2+x0) 10; draw_string (string_of_int v)


let balls = signal ()
let arrived = signal ()

let display () =
  let cnts = Array.create nb_cnts 0
  in
  let rec display_loop () =
    await balls
      (fun l -> clear_graph ();
	List.iter
	  (fun (Pos(x,y)) -> draw_ball x y)
	  l;
	for i = 0 to nb_cnts-1 do
	  draw_cnt i cnts.(i)
	done;
	draw_pts ();
	synchronize ();
	display_loop ()
      )
  in

  let rec cnts_loop () =
    await arrived
      (fun l ->
	List.iter
	  (fun (Int x) -> 
	    let n = (x-x0/2)/(ystep*2) in cnts.(n) <- cnts.(n) + 1)
	  l;
	cnts_loop ())
  in

  tail_par
    display_loop
    cnts_loop


let rec ball2 x y () =
  if y = -radius then begin
    emit arrived (Int x);
    term ()
  end
  else begin 
    emit balls (Pos (x,y));
    pause (ball2 x (y-1))
  end

let rec ball x y dx dy () =
  let x = x+dx
  and y = y+dy
  in
  if y=y0-ystep+rr then
    ball2 x y ()
  else begin
    emit balls (Pos (x,y));
    let dx = 
      if (y-y0-rr) mod ystep = 0 then
	if Random.int 2 = 0 then -1 else 1
      else dx
    in
    pause (ball x y dx dy)
  end


let rec gen () =
  let rec wait n k () =
    if n = 0 then k ()
    else pause (wait (n-1) k)
  in
  tail_par 
    (ball xinit yinit 0 (-1)) 
    (wait gen_itv gen)


let main () =
  open_graph (":0 " ^ string_of_int (x0+ystep*nb_cnts*2) 
	      ^ "x" ^ string_of_int (y0+ystep*h));
  set_window_title "Galton";
  make_pts ();
  auto_synchronize false;
  tail_par
    gen
    display
;;

start [ main ];;
