let run p = p ();;

type coord = {
    x : float;
    y : float;
    z : float;
  }
;;

type planet = {
    id : int;
    masse : float;
    pos : coord;
    v : coord;
  }
;;


module SV =
  struct
    type t = SV_Null
      | SV_Planet of planet
      | SV_Coord of coord
      | SV_Click of (int * int)
      | SV_Key of char
  end

module TML (*: Tml.TML_TYPE *) = Tml.Gen_TML(SV);;
open TML
open SV

let nothing = ()

(* constantes *)
let g = 6.67 (*10^-11*)
;;
let dt = 0.1
;;

(* --------------------------------------------------------------------- *)
(* fonctions aux *)
let v_init() =
  {x = (Random.float 100.0) -. 50.0;
   y = (Random.float 100.0) -. 50.0;
   z = (Random.float 100.0) -. 50.0;}
;;

let new_pos x y =
  let max_x_2 = (Graphics.size_x()) / 2 in
  let max_y_2 = (Graphics.size_y()) / 2 in
  {x = float_of_int (x - max_x_2);
   y = float_of_int (y - max_y_2);
   z = (Random.float 200.0) -. 100.0;}
;;

let color_of_int = function
  | 0 -> Graphics.yellow
  | 1 -> Graphics.blue
  | 2 -> Graphics.green
  | 3 -> Graphics.red
  | 4 -> Graphics.cyan
  | 5 -> Graphics.black
  | 6 -> Graphics.magenta
  | _ -> Graphics.black
;;
(* --------------------------------------------------------------------- *)

(* fonction de maj *)
let compute_pos =
  let distance2 pos1 pos2 =
    (pos2.x -. pos1.x)*.(pos2.x -. pos1.x)
      +. (pos2.y -. pos1.y)*.(pos2.y -. pos1.y)
      +. (pos2.z -. pos1.z)*.(pos2.z -. pos1.z)
  in
  let force p1 p2 =
    let d2 = distance2 p1.pos p2.pos in
    let d = sqrt d2 in
    if (d <> 0.0) then 
      let  f12 = g *. (p1.masse *. p2.masse) /. d2 in
      (f12 *. (p2.pos.x -. p1.pos.x) /. d,
       f12 *. (p2.pos.y -. p1.pos.y) /. d,
       f12 *. (p2.pos.z -. p1.pos.z) /. d)
    else
      (0.0, 0.0, 0.0)
  in
  fun me all ->
    let fx, fy, fz = 
      (List.fold_left 
	 (fun (fx,fy,fz) p -> 
	   let x,y,z = force me p in
	   (fx +. x), 
	   (fy +. y), 
	   (fz +. z)) 
	 (0.0, 0.0, 0.0) 
	 all)
    in
    let v = {x = (me).v.x +. fx *. dt;
	     y = (me).v.y +. fy *. dt;
	     z = (me).v.z +. fz *. dt;}
    in
    let pos = {x = (me).pos.x +. v.x *. dt;
	       y = (me).pos.y +. v.y *. dt;
	       z = (me).pos.z +. v.z *. dt;}
    in
    {id = (me).id;
     masse = (me).masse;
     pos = pos;
     v = v;}
;;

let maj_aff all =
  Graphics.clear_graph();
  let max_x_2 = (Graphics.size_x()) / 2 in
  let max_y_2 = (Graphics.size_y()) / 2 in
  List.iter 
    (fun p -> 
      Graphics.set_color (color_of_int (p.id mod 7));
      Graphics.fill_circle 
	(int_of_float p.pos.x + max_x_2)
	(int_of_float p.pos.y  + max_y_2)
	(*3*)
	(if (-200.0 < p.pos.z) & (p.pos.z < 5000.0) then 
	  5+(int_of_float p.pos.z / 50) 
	else 1))
    all
;;

(****************************************************************************)
let planet id masse pos_init v_init p () =
  let me = ref {id = id; masse = masse; pos = pos_init; v = v_init} in
  let rec loop () =
    emit p (SV_Planet !me);
    await p 
      (fun all -> me := compute_pos !me 
	  (List.map (fun (SV_Planet p) -> p) all); loop ())
  in loop ()
;;

let soleil id masse pos_init p () =
  let me = {id = id; masse = masse; pos = pos_init; v = {x=0.0;y=0.0;z=0.0}} in
  let rec loop () =
    emit p (SV_Planet me); pause loop
  in loop ()
;;

(* Interface graphique *)
let click_of_button_down click () =
  let rec loop () =
    if Graphics.button_down() then emit click (SV_Click (Graphics.mouse_pos()));
    pause loop
  in loop ()

let key_of_key_pressed key () =
  let rec loop () =
    if Graphics.key_pressed() then emit key (SV_Key (Graphics.read_key()));
    pause loop
  in loop ()

let fenetre new_planet p kill_sun kill suspend () =
  let read_click () = 
    let click = signal () in
    tail_par
      (click_of_button_down click)
      (let rec loop () =
	present click 
	  (fun () -> pause loop)
	  (fun () -> await_immediate_one click 
	      (fun (SV_Click (x,y)) -> emit new_planet (SV_Coord (new_pos x y));
		pause loop))
      in loop)
  in
  let read_key () = 
    let key = signal () in
    tail_par
      (key_of_key_pressed key)
      (let rec loop () =
	await_immediate_one key 
	  (fun (SV_Key c) -> print_string ("key_pressed = ");print_char c;print_newline();
	    begin
	      match c with
	      | 's' -> emit kill_sun SV_Null
	      | 'k' -> emit kill SV_Null
	      | 'p' -> emit suspend SV_Null
	      | 'q' -> exit 0
	      | _ -> nothing
	    end;
	    pause loop)
      in loop)
  in
  let aff = signal () in
  Graphics.open_graph "";
  Graphics.auto_synchronize false;
  tail_parn
    [
     (let rec loop () =
       await p (fun all -> 
	 maj_aff (List.map (fun (SV_Planet p) -> p) all);
	 Graphics.synchronize();
	 pause (fun () -> pause loop))
     in loop)
       ;
     read_click 
       ;
     read_key 
   ]
;;

(* TML : ce processus difficile  traduire car ambiguits syntaxiques ! *)
let rec add_aux id new_planet kill kill_new p () =
  await new_planet (fun pos -> 
    let pos = List.map (fun (SV_Coord c) -> c) pos in
    tail_par
      (fun () -> 
	dountil kill_new (planet id 1.0 (List.hd pos) (v_init()) p) term)
      
      (fun () -> let new_kill_new = signal () in
      tail_par
	(add_aux (id+1) new_planet kill new_kill_new p)
	(fun () -> par
	    (await new_planet)
	    (await_immediate kill_new)
	    (fun () -> await kill (fun _ -> emit new_kill_new SV_Null; Done)))))

let add id new_planet kill p () =
  run (add_aux id new_planet kill kill p)

let systeme () =
  let p = signal () in
  let new_planet = signal () in
  let kill_sun = signal () in
  let kill = signal () in
  let suspend = signal () in
  let active = signal () in
  tail_par
    (fenetre new_planet p kill_sun kill suspend)
    
    (fun () -> controlwith suspend
	(fun () -> tail_parn
	   [
	    (fun () -> dountil kill_sun (soleil 0 30000.0 {x=0.0;y=0.0;z=0.0} p) term)
	      ;
	    (planet 1 1.0
	       {x = 0.0; y = -. 200.0; z = 0.0} 
	       {x = 30.0; y = 0.0; z = 0.0}
               p)
	     ;
	    (planet 2 1.0
	       {x = 0.0; y = 200.0; z = 0.0} 
               {x = -. 30.0; y = 0.0; z = 0.0}
	       p)
	      ;
	    (planet 3 1.0
	       {x = 200.0; y = 0.0; z = 0.0} 
	       {x = 0.0; y = 0.0; z = 30.0}
	       p)
	      ;
	    (add 4 new_planet kill p)
	  ]
	) term)
;;

(****************************************************************************)
let _ = Random.self_init() 
;;
let main = systeme
;;

start [ systeme ]
