
type sv_t =
    Pos of int*int*int
  | Click of int *int

module TML = Tml.Gen_TML(struct type t = sv_t end)

open TML
open Graphics

let max_radius = 7

let xg = 600
let yg = 600

let xm = ref (xg/2)
let ym = ref (yg/2)

let click = signal ()
let zombies_pos = signal ()

let pi  = 3.1415927
let pi2 = pi /. 2.

let angle dx dy =
  if dx = 0 then if dy>0 then pi2 else -.pi2
  else let a = atan ((float_of_int dy) /. (float_of_int dx)) in
  if dx > 0 then a+.pi else a


let try_update dx dy x y =
  if pre click then 
    let t = angle (!xm - (int_of_float !x)) (!ym - (int_of_float !y)) in
    dx := cos t;
    dy := sin t

let update_radius c r =
  if !c then if !r>max_radius then c:=false else r:=!r+1 
  else if !r<2 then c:=true else r:=!r-1

let create_zombi x y =
  let x = ref (float_of_int x)
  and y = ref (float_of_int y)
  and r = ref ((Random.int max_radius)+1)
  and c = ref true in
  let t = angle (!xm - (int_of_float !x)) (!ym - (int_of_float !y)) in
  let dx = ref (cos t)
  and dy = ref (sin t)
  in
  let rec loop () =
    try_update dx dy x y;
    x := !x -. !dx;
    y := !y -. !dy;
    update_radius c r;
    emit zombies_pos (Pos((int_of_float !x, int_of_float !y, !r)));
    pause loop
  in
  loop

let rec new_pos () =
  await_one click (fun (Click(x,y)) ->
    xm:=x; ym:=y;
    new_pos ())
    

let read_click click () =
  let rec loop () =
    let pos_x, pos_y = Graphics.mouse_pos() in
    if Graphics.button_down() then emit click (Click (pos_x,pos_y));
    pause loop
  in
  loop ()

let draw_zombi x y r = draw_circle x y r

let arrived = signal ()

let display () =
  let rec display_loop () =
    await zombies_pos
      (fun l -> clear_graph ();
	List.iter
	  (fun (Pos(x,y,r)) -> draw_zombi x y r)
	  l;
	synchronize ();
	display_loop ()
      )
  in
  display_loop ()

let gen () =
  tail_fordopar 1 1000 
    (fun i -> let x = Random.int xg and y = Random.int yg in
    create_zombi x y ())
;;

let main () =
  open_graph (":0 " ^ (string_of_int xg)
	      ^ "x" ^ (string_of_int yg));
  set_window_title "Zombies";
  auto_synchronize false;
  tail_parn
    [
     gen;
     display;
     read_click click;
     new_pos
   ]
;;

start [ main ];;
